@@ -2,8 +2,8 @@
 #
 # arithscan.R
 #
-# copyright (c) 2005-2014, Karl W Broman
-# last modified Oct, 2014
+# copyright (c) 2005-2019, Karl W Broman
+# last modified Dec, 2019
 # first written Mar, 2005
 #
 #     This program is free software; you can redistribute it and/or
@@ -28,29 +28,19 @@
 "-.scanone" <-
     function(e1,e2)
 {
-    if(!any(class(e1) == "scanone"))
+    if(!inherits(e1, "scanone"))
         stop("Input should have class \"scanone\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
     if(missing(e2)) {
         class(e1) <- "data.frame"
         e1[,-(1:2)] <- -e1[,-(1:2)]
         class(e1) <- c("scanone","data.frame")
-        if(!is.null(df1))
-            attr(e1, "df") <- -df1
 
         return(e1)
     }
-    if(!any(class(e2) == "scanone"))
+    if(!inherits(e2, "scanone"))
         stop("Input should have class \"scanone\".")
 
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     class(e1) <- class(e2) <- "data.frame"
     if(nrow(e1) != nrow(e2)) {
         u1 <- levels(e1[,1])
@@ -75,34 +65,19 @@
 
     class(e1) <- c("scanone","data.frame")
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 - df2
-    }
-
     e1
 }
 
 "+.scanone" <-
     function(e1,e2)
 {
-    if(!any(class(e1) == "scanone"))
+    if(!inherits(e1, "scanone"))
         stop("Input should have class \"scanone\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
     if(missing(e2)) return(e1)
-    if(!any(class(e2) == "scanone"))
+    if(!inherits(e2, "scanone"))
         stop("Input should have class \"scanone\".")
 
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     class(e1) <- class(e2) <- "data.frame"
     if(nrow(e1) != nrow(e2)) {
         u1 <- levels(e1[,1])
@@ -121,26 +96,15 @@
     e1[,3:nc] <- e1[,3:nc] + e2[,3:nc]
     class(e1) <- c("scanone","data.frame")
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 + df2
-    }
-
     e1
 }
 
 "-.scanoneperm" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scanoneperm"))
+    if(!inherits(e1, "scanoneperm"))
         stop("Input should have class \"scanoneperm\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
     if(missing(e2)) {
         e1.x <- ("xchr" %in% names(attributes(e1)))
         if(e1.x) {
@@ -153,18 +117,11 @@
             class(e1) <- theclass
         }
 
-        if(!is.null(df1))
-            attr(e1, "df") <- -df1
-
         return(e1)
     }
-    if(!any(class(e2) == "scanoneperm"))
+    if(!inherits(e2, "scanoneperm"))
         stop("Input should have class \"scanoneperm\".")
 
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     # check input
     e1.x <- ("xchr" %in% names(attributes(e1)))
     e2.x <- ("xchr" %in% names(attributes(e2)))
@@ -194,33 +151,18 @@
         class(e1) <- theclass
     }
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 - df2
-    }
-
     e1
 }
 
 "+.scanoneperm" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scanoneperm"))
+    if(!inherits(e1, "scanoneperm"))
         stop("Input should have class \"scanoneperm\".")
     if(missing(e2)) return(e1)
-    if(!any(class(e2) == "scanoneperm"))
+    if(!inherits(e2, "scanoneperm"))
         stop("Input should have class \"scanoneperm\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     # check input
     e1.x <- ("xchr" %in% names(attributes(e1)))
     e2.x <- ("xchr" %in% names(attributes(e2)))
@@ -242,13 +184,6 @@
         class(e1) <- theclass
     }
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 + df2
-    }
-
     e1
 }
 
@@ -258,29 +193,19 @@
 "-.scantwo" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scantwo"))
+    if(!inherits(e1, "scantwo"))
         stop("Input should have class \"scantwo\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
     if(missing(e2)) {
         e1$lod <- -e1$lod
         if("scanoneX" %in% names(e1))
             e1$scanoneX <- -e1$scanoneX
-        if(!is.null(df1))
-            attr(e1, "df") <- -df1
 
         return(e1)
     }
-    if(!any(class(e2) == "scantwo"))
+    if(!inherits(e2, "scantwo"))
         stop("Input should have class \"scantwo\".")
 
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     e1x <- "scanoneX" %in% names(e1)
     e2x <- "scanoneX" %in% names(e2)
 
@@ -295,13 +220,6 @@
             e1$scanoneX <- e1$scanoneX - e2$scanoneX
     }
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 - df2
-    }
-
     e1
 }
 
@@ -312,20 +230,12 @@
 "+.scantwo" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scantwo"))
+    if(!inherits(e1, "scantwo"))
         stop("Input should have class \"scantwo\".")
     if(missing(e2)) return(e1)
-    if(!any(class(e2) == "scantwo"))
+    if(!inherits(e2, "scantwo"))
         stop("Input should have class \"scantwo\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     e1x <- "scanoneX" %in% names(e1)
     e2x <- "scanoneX" %in% names(e2)
 
@@ -340,13 +250,6 @@
             e1$scanoneX <- e1$scanoneX + e2$scanoneX
     }
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 + df2
-    }
-
     e1
 }
 
@@ -357,13 +260,9 @@
 "-.scantwoperm" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scantwoperm"))
+    if(!inherits(e1, "scantwoperm"))
         stop("Input should have class \"scantwoperm\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
     if(missing(e2)) {
         # x-chr-specific
         if("AA" %in% names(e1)) {
@@ -375,19 +274,13 @@
 
         for(i in 1:length(e1))
             e1[[i]] <- -e1[[i]]
-        if(!is.null(df1))
-            attr(e1, "df") <- -df1
 
         return(e1)
     }
 
-    if(!any(class(e2) == "scantwoperm"))
+    if(!inherits(e2, "scantwoperm"))
         stop("Input should have class \"scantwoperm\".")
 
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     # x-chr-specific
     if("AA" %in% names(e1) || "AA" %in% names(e2)) {
         if(!("AA" %in% names(e1) && "AA" %in% names(e2)))
@@ -410,13 +303,6 @@
     for(i in 1:length(e1))
         e1[[i]] <- e1[[i]] - e2[[i]]
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 - df2
-    }
-
     e1
 }
 
@@ -427,20 +313,12 @@
 "+.scantwoperm" <-
     function(e1, e2)
 {
-    if(!any(class(e1) == "scantwoperm"))
+    if(!inherits(e1, "scantwoperm"))
         stop("Input should have class \"scantwoperm\".")
     if(missing(e2)) return(e1)
-    if(!any(class(e2) == "scantwoperm"))
+    if(!inherits(e2, "scantwoperm"))
         stop("Input should have class \"scantwoperm\".")
 
-    if("df" %in% names(attributes(e1)))
-        df1 <- attr(e1, "df")
-    else df1 <- NULL
-
-    if("df" %in% names(attributes(e2)))
-        df2 <- attr(e2, "df")
-    else df2 <- NULL
-
     # x-chr-specific
     if("AA" %in% names(e1) || "AA" %in% names(e2)) {
         if(!("AA" %in% names(e1) && "AA" %in% names(e2)))
@@ -463,13 +341,6 @@
     for(i in 1:length(e1))
         e1[[i]] <- e1[[i]] + e2[[i]]
 
-    if(!is.null(df1) && !is.null(df2)) {
-        if(length(df1) != length(df2))
-            warning("Dimensions of degrees of freedom don't match; this may indicate a problem.")
-        else
-            attr(e1, "df") <- df1 + df2
-    }
-
     e1
 }
 
