@@ -2,8 +2,8 @@
 #
 # est.rf.R
 #
-# copyright (c) 2001-2016, Karl W Broman
-# last modified Jan, 2016
+# copyright (c) 2001-2022, Karl W Broman
+# last modified Jun, 2022
 # first written Apr, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -33,7 +33,7 @@
 est.rf <-
     function(cross, maxit=10000, tol=1e-6)
 {
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("Input should have class \"cross\".")
 
     n.chr <- nchr(cross)
@@ -41,8 +41,8 @@ est.rf <-
     n.ind <- nind(cross)
     mar.names <- unlist(lapply(cross$geno,function(a) colnames(a$data)))
 
-    type <- class(cross)[1]
-    chrtype <- sapply(cross$geno,class)
+    type <- crosstype(cross)
+    chr_type <- sapply(cross$geno, chrtype)
 
     is.bcsft <- (type == "bcsft")
     if(is.bcsft) {
@@ -58,7 +58,7 @@ est.rf <-
         temp <- cross$geno[[i]]$data
 
         # treat X chromosome specially in an intercross or BCsFt with t>0.
-        if((type=="f2" || is.bcsft) && chrtype[i]=="X") {
+        if((type=="f2" || is.bcsft) && chr_type[i]=="X") {
             fixX <- TRUE
             if(i != 1) xchrcol <- c(xchrcol,ncol(Geno)+(1:ncol(cross$geno[[i]]$data)))
             else xchrcol <- 1:ncol(cross$geno[[i]]$data)
@@ -79,7 +79,7 @@ est.rf <-
         cfunc <- "est_rf_4way"
     else if(type=="ri8sib" || type=="ri8self" || type=="ri4sib" || type=="ri4self") {
         cfunc <- paste("est_rf_", type, sep="")
-        if(any(chrtype == "X"))
+        if(any(chr_type == "X"))
             warning("est.rf not working properly for the X chromosome for 4- or 8-way RIL.")
     }
     else if(type == "bcsft")
@@ -156,14 +156,14 @@ est.rf <-
 
 
 
-plotRF <- plot.rf <-
+plotRF <-
     function(x, chr, what=c("both","lod","rf"),
              alternate.chrid=FALSE, zmax=12,
              mark.diagonal=FALSE,
              col.scheme=c("viridis", "redblue"),
              ...)
 {
-    if(!any(class(x) == "cross"))
+    if(!inherits(x, "cross"))
         stop("Input should have class \"cross\".")
 
     what <- match.arg(what)
@@ -332,22 +332,22 @@ plotRF <- plot.rf <-
 checkAlleles <-
     function(cross, threshold=3, verbose=TRUE)
 {
-    if(!any(class(cross) == "cross"))
+    if(!inherits(cross, "cross"))
         stop("checkAlleles() only works for cross objects.")
 
-    type <- class(cross)[1]
+    type <- crosstype(cross)
     if(type != "f2" && type != "bc" &&
        type != "risib" && type != "riself" && type != "dh" && type!="haploid")
         stop("checkAlleles not available for cross type ", type, ".")
 
     # drop X chromosome
-    chrtype <- sapply(cross$geno,class)
-    if(all(chrtype=="X")) {
+    chr_type <- sapply(cross$geno, chrtype)
+    if(all(chr_type=="X")) {
         if(verbose) cat("checkAlleles() only works for autosomal data.\n")
         return(NULL)
     }
 
-    cross <- subset(cross, chr = (chrtype != "X"))
+    cross <- subset(cross, chr = (chr_type != "X"))
 
     n.mar <- nmar(cross)
     mar.names <- unlist(lapply(cross$geno,function(a) colnames(a$data)))
@@ -356,6 +356,12 @@ checkAlleles <-
         warning("First running est.rf.")
         cross <- est.rf(cross)
     }
+    onlylod <- attr(cross$rf, "onlylod")
+    if(!is.null(onlylod) && onlylod) { # need to re-run est.rf()
+        warning("First running est.rf.")
+        cross <- est.rf(cross)
+    }
+
     diag(cross$rf) <- 0
     lod <- rf <- cross$rf
     lod[lower.tri(lod)] <- t(lod)[lower.tri(lod)]
@@ -392,7 +398,7 @@ checkAlleles <-
 pull.rf <-
     function(cross, what=c("rf", "lod"), chr)
 {
-    if(!("cross" %in% class(cross)))
+    if(!inherits(cross, "cross"))
         stop("Input must have class \"cross\".")
 
     if(!missing(chr)) cross <- subset(cross, chr=chr)
@@ -432,7 +438,7 @@ pull.rf <-
 plot.rfmatrix <-
     function(x, marker, ...)
 {
-    if(!("rfmatrix" %in% class(x)))
+    if(!inherits(x, "rfmatrix"))
         stop("Input must have class \"rfmatrix\".")
 
     if(missing(marker))
