@@ -2,8 +2,8 @@
 #
 # scanone.R
 #
-# copyright (c) 2001-2015, Karl W Broman
-# last modified Oct, 2015
+# copyright (c) 2001-2020, Karl W Broman
+# last modified Feb, 2020
 # first written Feb, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -48,9 +48,10 @@ scanone <-
     use <- match.arg(use)
 
     # in RIL, treat X chromomse like an autosome
-    chrtype <- sapply(cross$geno, class)
-    if(any(chrtype=="X") && (class(cross)[1] == "risib" || class(cross)[1] == "riself"))
-        for(i in which(chrtype=="X")) class(cross$geno[[i]]) <- "A"
+    chr_type <- sapply(cross$geno, chrtype)
+    crosstype <- crosstype(cross)
+    if(any(chr_type=="X") && (crosstype == "risib" || crosstype == "riself"))
+        for(i in which(chr_type=="X")) class(cross$geno[[i]]) <- "A"
 
     if(!missing(n.perm) && n.perm > 0 && n.cluster > 1) {
         cat(" -Running permutations via a cluster of", n.cluster, "nodes.\n")
@@ -279,7 +280,7 @@ scanone <-
     n.chr <- nchr(cross)
     n.ind <- nind(cross)
     n.phe <- ncol(pheno)
-    type <- class(cross)[1]
+    type <- crosstype(cross)
     is.bcs <- FALSE
     if(type == "bcsft") {
         cross.scheme <- attr(cross, "scheme")
@@ -365,8 +366,8 @@ scanone <-
 
     # scan genome one chromosome at a time
     for(i in 1:n.chr) {
-        chrtype <- class(cross$geno[[i]])
-        if(chrtype=="X") {
+        chr_type <- chrtype(cross$geno[[i]])
+        if(chr_type=="X") {
             sexpgm <- getsex(cross)
             ac <- revisecovar(sexpgm,addcovar)
 
@@ -396,7 +397,7 @@ scanone <-
         }
 
         # get genotype names
-        gen.names <- getgenonames(type,chrtype,"full",sexpgm,attributes(cross))
+        gen.names <- getgenonames(type,chr_type,"full",sexpgm,attributes(cross))
         n.gen <- length(gen.names)
 
         # starting values for interval mapping
@@ -421,7 +422,7 @@ scanone <-
             if(type=="4way") newgeno[newgeno>4] <- 0
 
             # revise X chromosome genotypes
-            if(chrtype=="X" && (type %in% c("bc","f2","bcsft")))
+            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                 newgeno <- reviseXdata(type, "full", sexpgm, geno=newgeno,
                                        cross.attr=attributes(cross))
 
@@ -445,7 +446,7 @@ scanone <-
             n.draws <- dim(draws)[3]
 
             # revise X chromosome genotypes
-            if(chrtype=="X" && (type %in% c("bc","f2","bcsft")))
+            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                 draws <- reviseXdata(type, "full", sexpgm, draws=draws,
                                      cross.attr=attributes(cross))
 
@@ -477,7 +478,7 @@ scanone <-
             n.pos <- ncol(genoprob)
 
             # revise X chromosome genotypes
-            if(chrtype=="X" && (type %in% c("bc","f2","bcsft")))
+            if(chr_type=="X" && (type %in% c("bc","f2","bcsft")))
                 genoprob <- reviseXdata(type, "full", sexpgm, prob=genoprob,
                                         cross.attr=attributes(cross))
 
@@ -722,7 +723,7 @@ scanone <-
         }
 
         # get null log10 likelihood for the X chromosome
-        if(chrtype=="X") {
+        if(chr_type=="X") {
 
             # determine which covariates belong in null hypothesis
             temp <- scanoneXnull(type, sexpgm, cross.attr=attributes(cross))
@@ -824,7 +825,7 @@ scanone <-
             w[o] <- paste("c",names(cross$geno)[i],".",w[o],sep="")
 
         z <- as.data.frame(z, stringsAsFactors=TRUE)
-        z <- cbind(chr=rep(names(cross$geno)[i],length(map)),
+        z <- cbind(chr=factor(rep(names(cross$geno)[i],length(map)),levels=names(cross$geno)),
                    pos=as.numeric(map), z)
         rownames(z) <- w
 
@@ -866,7 +867,7 @@ scanone.perm <-
         addcovar <- intcovar <- NULL
     }
 
-    chr.type <- sapply(cross$geno, class)
+    chr.type <- sapply(cross$geno, chrtype)
     if((all(chr.type=="X") || all(chr.type=="X")) && perm.Xsp==TRUE)
         warning("All chromosomes of the same type, so X-chr specific permutations not needed.\n")
 
@@ -910,7 +911,7 @@ scanone.perm <-
 
     attr(res,"method") <- method
     attr(res,"model") <- model
-    attr(res,"type") <- class(cross)[1]
+    attr(res,"type") <- crosstype(cross)
 
     if(any(chr.type=="X") && any(chr.type=="A") && perm.Xsp)
         class(res) <- c("scanoneperm", "list")
@@ -948,11 +949,11 @@ scanone.perm.engine <-
     if( (n.phe==1) && ((method=="imp") || (method=="hk")) &&
        model == "normal" &&
        is.null(addcovar) && is.null(intcovar) ) {
-        chrtype <- sapply(cross$geno, class)
+        chr_type <- sapply(cross$geno, chrtype)
         sexpgm <- getsex(cross)
         sex <- sexpgm$sex
         pgm <- sexpgm$pgm
-        if(all(chrtype=="A"))
+        if(all(chr_type=="A"))
             batch.mode <- TRUE
         else if((is.null(sex) || length(unique(sex))==1) &&
                 (is.null(pgm) || length(unique(pgm))==1))
